﻿// PARTICLE EXPERIMENTS
// Author : Seb Lee-Delisle
// Blog : www.sebleedelisle.com
// Company : www.pluginmedia.net
//
// This work is licensed under a Creative Commons  2.0 License.

// Full details at 
// http://creativecommons.org/licenses/by/2.0/uk/

// You may re-use this code as you wish, but a credit would be 
// appreciated. And I'd love to see what you do with it! 
// mail me : seb@sebleedelisle.com

package{
	

import flash.filters.*;
import flash.display.*; 
import flash.geom.*;

public class Particle3D { 

	public var xVel:Number;
	public var yVel:Number;
	public var zVel:Number;
	
	public var x:Number;
	public var y:Number;
	public var z:Number
	public var scale:Number;
	
	public var drag:Number = 1;
	public var gravity:Number = 0;
	
	static var fov:Number = 250;	
	public var bmd:BitmapData;
	public var targetBitmap : BitmapData; 
	
	public var m : Matrix = new Matrix(); 
	
	public var halfWidth : int; 
	public var halfHeight : int; 
	

	public function Particle3D(bmd:BitmapData, targetbitmap:BitmapData, xpos:Number, ypos:Number, zpos:Number){


		this.bmd = bmd; 
		this.targetBitmap = targetbitmap;  

		halfWidth = targetBitmap.width/2; 
		halfHeight = targetBitmap.height/2; 
		

		reset(xpos, ypos, zpos); 
		
		
	

	}
	
	public function reset(xpos : Number, ypos : Number, zpos : Number) : void
	{
		x = xpos;
		y = ypos;
		z = zpos;
		scale = 0.11; 
		xVel = 0;
		yVel = 0;
		zVel = 0;
		
		render();
	}
	
	public function setVel(xvel:Number, yvel:Number, zvel:Number){
		xVel = xvel;
		yVel = yvel;
		zVel = zvel;
	}
	
	
	public function update(){
		
		// add the velocity to the particle's position... 
		x += xVel;
		y += yVel;
		z += zVel;
		
		// apply drag
		xVel*=drag;
		yVel*=drag;
		zVel*=drag;		
		
		// gravity
		yVel+=gravity;
		
	}
	
	
	public function render(){

		var newscale:Number = fov/(fov+z);
		var scalefactor : Number = newscale*scale;
		
		m.identity(); 
		m.scale(scalefactor, scalefactor); 
		m.translate(halfWidth + (x*newscale), halfHeight + (y*newscale)); 
		
		targetBitmap.draw(bmd, m, null, null, null, true); 
		

	}
	
	public function rotateY(value:Number){

		var cosRY:Number = Math.cos(value* Math.PI / 180);
		var sinRY:Number = Math.sin(value* Math.PI / 180);
		
		var tempX:Number = x;
		x= (tempX*cosRY)+(z*sinRY);
		z= (tempX*-sinRY)+(z*cosRY);
		
		tempX = xVel;
		
		xVel= (tempX*cosRY)+(zVel*sinRY);
		zVel= (tempX*-sinRY)+(zVel*cosRY);


	}
	
	public function toString() : String
	{
		return("["+x+" "+y+" "+z+" "+xVel+" "+ yVel+" "+ zVel)
		
		
	}
	

}
}